import winim
import os
import strutils
import strformat
import random
import math

randomize()

proc soundOpen(sfile: string): string =
    var sAlias: string
    for i in 0..9:
        sAlias &= char(rand(65..90))
    let err = mciSendStringW(fmt("open \"{sfile}\" alias {sAlias}"), "", 0, 0)
    if err != 0: return ""
    mciSendStringW(fmt("set {sAlias} time format milliseconds"), "", 0, 0)
    return  sAlias

proc soundClose(sAlias: string): bool {.discardable.} =
    if sAlias == "": return false
    return mciSendStringW(fmt("close {sAlias}"), "", 0, 0) == 0

proc soundSeek(sAlias: string, iPos: int = 0): bool {.discardable.} = 
    if sAlias == "": return false
    return mciSendStringW(fmt("seek {sAlias} to ") & (if iPos > 0: $iPos else: "start"), "", 0, 0) == 0

proc soundLength(sAlias: string): int =
    if sAlias == "": return -1
    var lpszReturn: LPWSTR = newWideCString("", 10)
    mciSendStringW(fmt("status {sAlias} length"), lpszReturn, 255, 0)
    result = parseInt($lpszReturn)

proc soundPos(sAlias: string): int =
    if sAlias == "": return -1
    var lpszReturn: LPWSTR = newWideCString("", 10)
    mciSendStringW(fmt("status {sAlias} position"), lpszReturn, 255, 0)
    result = parseInt($lpszReturn)

proc soundPlay(sAlias: string, bWait: bool = false): bool {.discardable.} =
    if sAlias == "": return false
    if soundPos(sAlias) >= soundLength(sAlias):
        discard soundSeek(sAlias)
    return mciSendStringW(fmt("play {sAlias}") & (if bWait: " wait" else: ""), "", 0, 0) == 0

proc soundPause(sAlias: string): bool {.discardable.} = 
    if sAlias == "": return false
    return mciSendStringW(fmt("pause {sAlias}"), "", 0, 0) == 0

proc soundResume(sAlias: string): bool {.discardable.} = 
    if sAlias == "": return false
    return mciSendStringW(fmt("resume {sAlias}"), "", 0, 0) == 0

proc soundStop(sAlias: string): bool {.discardable.} = 
    if sAlias == "": return false
    return mciSendStringW(fmt("stop {sAlias}"), "", 0, 0) == 0

proc soundStatus(sAlias: string): string =
    if sAlias == "": return "no alias"
    var lpszReturn: LPWSTR = newWideCString("", 10)
    mciSendStringW(fmt("status {sAlias} mode"), lpszReturn, 255, 0)
    return $lpszReturn

proc soundSetVolume(sAlias: string, iVolume: int): bool {.discardable.} =
    if sAlias == "": return false
    if iVolume < 0 or iVolume > 1000: return false
    return mciSendStringW(fmt("setaudio {sAlias} volume to {iVolume}"), "", 0, 0) == 0

proc soundTicksToTime(iTicks: int): string =
    if iTicks < 0: return "error"
    var m, h, s: int
    s = int(iTicks / 1000)
    h = int(s / 3600)
    s = floorMod(s, 3600)
    m = int(s / 60)
    s = floorMod(s, 60)
    if h > 0:
        result = fmt("{h:0>2}:{m:0>2}:{s:0>2}")
    else:
        result = fmt("{m:0>2}:{s:0>2}")


# Beispielcode
# ==============
# "soundOpen" muss mit dem Dateinamen aufgerufen werden
# alle anderen Funktionen werden dann mit dem Alias aufgerufen.
# Einschraenkungen:
# MP3s mit VBR-Kodierung funktionieren nicht!
# Bei WAV-Dateien funktioniert das aendern der Lautstaerke nicht!

let sSndfile = r"Asia - Heat Of The Moment.mp3" # Pfad zur mp3-Datei
let sMyAlias = soundOpen(sSndfile) # Datei oeffnen (Rueckgabe = zufaelliger Alias)
echo "Alias: " & sMyAlias
soundSetVolume(sMyAlias, 1000) # Lautstaerke festlegen (0 = aus bis 1000 = max.)
var iLength = soundLength(sMyAlias) # Laufzeit des Sounds holen
echo "Soundlength " & soundTicksToTime(iLength)
soundPlay(sMyAlias) # Sound abspielen
var iPosition: int
while iPosition < iLength:
    sleep(1000)
    iPosition = soundPos(sMyAlias) # momentane Position auslesen
    echo "Position " & soundTicksToTime(iPosition) # in (hh:)mm:ss umwandeln
soundClose(sMyAlias) # Sound schliessen
